import submittingOrderView from "./views/submittingOrderView.js";
import connectWalletView from "./views/connectWalletView.js";
import axios from "https://cdn.jsdelivr.net/npm/axios@1.3.5/+esm";


let exchangeRate;
let wallet;
const getExchangeRate = async () => {
    exchangeRate = (
        await axios.get(`${window.location.origin}/ada-exchange-rate`)
    ).data;
};

const getWalletCredentials = async () => {
    wallet = (await axios.get(`${window.location.origin}/wallet-credentials`))
        .data;
};

const handleWalletConnect = async function (walletName) {
    let walletSupported = connectWalletView.isSupported(walletName);
    if (walletSupported) {
        connectWalletView.renderWalletConnectingSpinner(walletName);
        document
            .querySelector("#cancel-connection")
            .addEventListener("click", function (e) {
                submittingOrderView?.renderMessage(
                    `Cancelling ${walletName} connection`
                );
                localStorage.clear("walletName");
                window.location.reload();
            });

        let walletEnabled = connectWalletView.enableWallet(walletName);
        if (walletEnabled) {
            let walletConnected = await connectWalletView.connectWallet(
                walletName
            );

            if (walletConnected) {
                connectWalletView.renderWalletConnect(walletName);
                document
                    .querySelector("#disconnect-wallet")
                    .addEventListener("click", function (e) {
                        submittingOrderView?.renderMessage(
                            `Disabling ${walletName}`
                        );
                        localStorage.clear("walletName");
                        window.location.reload();
                    });
            }
        } else {
            connectWalletView.disableCheckboxes();
            connectWalletView.renderError(walletName + " wallet not installed");
            localStorage.clear("walletName");
            window.location.reload();
        }
    } else {
        connectWalletView.disableCheckboxes();
        connectWalletView.renderError(walletName + " not supported");
        localStorage.clear("walletName");
        window.location.reload();
    }
};

const controlWalletConnection = async function () {
    try {
        connectWalletView.walletCheckboxes.forEach(function (checkbox) {
            checkbox.addEventListener("change", function (e) {
                e.preventDefault();
                if (checkbox.checked == true) {
                    connectWalletView.persistWallet(checkbox.value);
                    handleWalletConnect(checkbox.value);
                }
            });
        });

        const storage = localStorage.getItem("walletName");
        if (storage) {
            handleWalletConnect(storage);
        }
    } catch (err) {
        console.log(err);
    }
};

const controlSubmittingOrder = async function () {
    try {
        // 1) Get order total and currency
        const submitteOrder = submittingOrderView.getSubmittedOrder();
        console.log({ submitteOrder });

        if (submitteOrder) {
            if (connectWalletView.api) {
                submittingOrderView.renderProcessing(wallet);

                // submit payment
                let amounts;
                //  = (submitteOrder.currency === 'ADA') ? parseInt(submitteOrder.total * 1000000) : null;
                if (submitteOrder.currency === "ADA") {
                    amounts = parseInt(
                        submitteOrder.total *
                            exchangeRate.usd_to_ada_rate *
                            1000000
                    );
                } else {
                    amounts = {
                        [submitteOrder.currency]: BigInt(submitteOrder.total),
                    };
                }

                let paymentSuccesful = await connectWalletView.payToAddress(
                    wallet.payment_address,
                    amounts
                );

                if (paymentSuccesful) {
                    submittingOrderView?.renderMessage("Payment successful");
                    window.location.reload();
                } else {
                    submittingOrderView.renderError(
                        "Order submission error, check network"
                    );
                    // window.location.reload();
                }
            } else {
                submittingOrderView.renderError("Wallet connection needed   ");
                window.location.reload();
            }
        } else {
            submittingOrderView.renderError("Order error");
            window.location.reload();
        }
    } catch (err) {
        console.log(err);
    }
};

const init = function () {
    getWalletCredentials();
    getExchangeRate();
    submittingOrderView.addHandlerRender(controlSubmittingOrder);
    connectWalletView.addHandlerRender(controlWalletConnection);
};
init();
